package webovyformular.dat_typy;

import java.util.*;

/**
 *
 *
 * <p>Description: </p> Tda slou pro prci se seznamem Znaku, tedy pro
 * objekty typu <b> ArrayList &lt;Znak&gt;</b>. Nem dn konstruktor a obsahuje
 * pouze statick tdy.
 *
 * <p>Copyright: Copyright (c) 2005</p>
 *
 *
 * @author Michal Hanzelka
 * @version 1.0
 */
public class Retezec {

  /**
   * Metoda toSting pevede <i>retezec</i> na etzec String, kter
   * bude obsahovat jmna Znak v <i>retezci</i>.
   * @param retezec ArrayList - etzec Znak
   * @return String - Stringov reprezentace <i>retezce</i>
   */
  public static String toString(ArrayList<Znak> retezec) {
    String string = "";

    for (Znak znak : retezec) {
      string += znak;
    }

    return string;
  }

  /**
   * Metoda toString pevede mnoinu <i>setRetezcu</i> na etzec String.
   * Vsledn String bude obsahovat nazatku znak <i>{</i> nsledovan
   * Stringovou reprezentac jednotolivch etzc ze <i>setRetezcu</i>
   * oddlench znakem <i>,</i> a bude ukonen znakem <i>}</i>.
   * @param setRetezcu HashSet - mnoina etzc Znak
   * @return String - Stringov reprezentace
   */
  public static String toString(HashSet<ArrayList<Znak>> setRetezcu) {
    String vysledek = "{";

    for (Iterator it = setRetezcu.iterator(); it.hasNext(); ) {
      vysledek += Retezec.toString( (ArrayList<Znak>) it.next());
      if (it.hasNext()) {
        vysledek += ", ";
      }
    }
    vysledek += "}";

    return vysledek;
  }

  /**
   * Metoda zretez zetez dva etzce (seznamy) zadan na vstupu. Prvky druhho
   * etzce se vlo v nezmnnm poad za posledn prvek prvnho etzce.
   * Je-li prvnm Znakem prvnho vstupnho etzce epsilon, je vrcen druh vstupn
   * etzec. Je-li prvnm Znakem druhho vstupn etzec epsilon, je vrcen prvn
   * vstupn etzec.
   * @param a ArrayList - prvn vstupn etzec Znak
   * @param b ArrayList - druh vstupn etzec Znak
   * @return ArrayList - vsledn etzec Znak vznikl zetzenm etzc
   * <i>a</i> a <i>b</i>
   */
  public static ArrayList<Znak> zretez(ArrayList<Znak> a, ArrayList<Znak> b) {

    ArrayList<Znak> vysledek = new ArrayList<Znak> ();

    if (a.get(0).isEpsilon()) { // eps (+) b = b
      return b;
    }
    else {
      if (b.get(0).isEpsilon()) { // a (+) eps = a
        return a;
      }
      else {
        vysledek.addAll(a);
        vysledek.addAll(b); // vysledek = a (+) b
        return vysledek;
      }
    }
  }

  /**
   * Metoda zretez zetz prvky mnoin zadanch na vstupu. Prvky vsledn
   * mnoiny jsou takov etzce, jejich prvn st je prvek mnoiny
   * <i>setA</i> a zbvajc st je prvek mnoiny <i>setB</i>. <br/>
   * Pklad: <br/>
   * <i>
   * setA = {abc,def} <br/>
   * setB = {uvw,xyz} <br/>
   * setAB = {abcuvw,abcxyz,defuvw,defxyz} <br/>
   * </i>
   * Je-li alespo jedna ze vstupnch mnoin przdn, je vsledn mnoina
   * rzdn.
   *
   * @param setA HashSet - mnoina etzc Znak
   * @param setB HashSet - mnoina etzc Znak
   * @return HashSet - vsledn mnoina etzc Znak
   */
  public static HashSet<ArrayList<Znak>> zretez(HashSet<ArrayList<Znak>> setA,
                                                HashSet<ArrayList<Znak>> setB) {

    HashSet<ArrayList<Znak>> setAB = new HashSet<ArrayList<Znak>> (); //vysledna mnozina

    if ( (!setA.isEmpty()) && (!setB.isEmpty())) {
      for (ArrayList<Znak> aktA : setA) { //pruchod pres vsechny retezce v setA
        for (ArrayList<Znak> aktB : setB) { //pruchod pres vsechny retezce v setB
          setAB.add(Retezec.zretez(aktA, aktB));
        }
      }
    }
    return setAB;

  }

  /**
   * Metoda zretez zetz etzec Znak s prvky mnoiny obsahujc etzce
   * Znak. Vsledn  mnoina obsahuje takov etzce, jejich prvn st je
   * zadan etzec <i>retA</i> a zbvajc st je nkter z prvk mnoiny
   * <i>setB</i> zadan na vstupu.
   * <br/>
   * Pklad: <br/>
   * <i>
   * retA = abc <br/>
   * setB = {uvw,xyz} <br/>
   * setAB = {abcuvw,abcxyz} <br/>
   * </i>
   *
   * @param retA ArrayList - etzec Znak
   * @param setB HashSet - mnoina etzc Znak
   * @return HashSet - vsledn mnoina etzc Znak
   */
  public static HashSet<ArrayList<Znak>> zretez(ArrayList<Znak> retA,
                                                HashSet<ArrayList<Znak>> setB) {

    HashSet<ArrayList<Znak>> setA = new HashSet<ArrayList<Znak>> ();
    setA.add(retA);
    return Retezec.zretez(setA, setB);
  }

  /**
   * Metoda zretez zetz dva zadan etzce Znak. Vznikne-li zetzenm
   * etzc <i>a</i> a <i>b</i> etzec del ne <i>k</i>, je vsledn etzec
   *  zkrcen na dlku <i>k</i>. <br/>
   * Pklad: <br/>
   * <i>
   * a = abc  <br/>
   * b = xyz  <br/>
   * k = 4    <br/>
   * vysledny_retezec = abcx
   * </i>
   *
   * @param a ArrayList - prvn etzec Znak
   * @param b ArrayList - druh etzec Znak
   * @param k int - parametr <i>k</i>
   * @return ArrayList - vsledn etzec Znak dlky nejve <i>k</i>
   * @throws NekladnyParamException kdy je parametr <i>k</i> nekladn slo
   */
  public static ArrayList<Znak> zretez(ArrayList<Znak> a, ArrayList<Znak> b,
                                       int k) throws NekladnyParamException {

    int i;
    ArrayList<Znak> vysledny_retezec = new ArrayList<Znak> ();

    if (k < 1) {
      throw new NekladnyParamException();
    }
    else {
      vysledny_retezec = Retezec.zretez(a, b);

      if (vysledny_retezec.size() > k) {
        for (i = vysledny_retezec.size() - 1; i >= k; i--) { //odstranovani symbolu od konce
          vysledny_retezec.remove(i);
        }
      }

      return vysledny_retezec;
    }
  }

  /**
   * Metoda zretez zetz prvky mnoin zadanch na vstupu. Prvky vsledn
   * mnoiny jsou takov etzce dlky maximln <i>k</i>, jejich prvn st
   * je prvek mnoiny <i>setA</i> a zbvajc st je prvek mnoiny
   * <i>setB</i>. <br/>
   * Pklad: <br/>
   * <i>
   * setA = {abc,def} <br/>
   * setB = {uvw,xyz} <br/>
   * k = 5
   * setAB = {abcuv,abcxy,defuv,defxy} <br/>
   * </i>
   * Je-li alespo jedna ze vstupnch mnoin przdn, je vsledn mnoina
   * przdn.
   *
   * @param setA HashSet - mnoina etzc Znak
   * @param setB HashSet - mnoina etzc Znak
   * @param k int - parametr <i>k</i>
   * @return HashSet - vsledn mnoina etzc Znak dlky nejve <i>k</i>
   * @throws NekladnyParamException kdy je parametr <i>k</i> nekladn slo
   */
  public static HashSet<ArrayList<Znak>> zretez(HashSet<ArrayList<Znak>> setA,
                                                HashSet<ArrayList<Znak>> setB,
                                                int k) throws
      NekladnyParamException {

    HashSet<ArrayList<Znak>> setAB = new HashSet<ArrayList<Znak>> (); //vysledna mnozina

    if ( (!setA.isEmpty()) && (!setB.isEmpty())) {
      for (ArrayList<Znak> aktA : setA) { //pruchod pres vsechny retezce v setA
        for (ArrayList<Znak> aktB : setB) { //pruchod pres vsechny retezce v setB
          setAB.add(Retezec.zretez(aktA, aktB, k));
        }
      }
    }
    return setAB;
  }

  /**
   * Metoda toRetezecZnaku pevede vstupn etzec String obsahujc jmna
   * symbol na odpovdajc seznam Znak typu ArrayList &lt;Znak&gt;.Tento
   * seznam je vrcen na vstup.
   * @param neterminaly HashSet - mnoina neterminl
   * @param terminaly HashSet - mnoina terminl
   * @param str String - etzec, kter m bt peveden na seznam Znak
   * @return ArrayList - vstupn seznam Znak
   * @throws ZnakyNerozeznanyException kdy zadan String obsahuje podetzec,
   * kter nen jmno terminlu ani neterminlu a ani jmenoEps
   * @throws ParamPrazdnyStringException kdy vstupn etzec je przdn nebo
   * obsahuje pouze etzec przdnch znak.
   */
  public static ArrayList<Znak> toRetezecZnaku(HashSet<Znak> neterminaly,
                                               HashSet<Znak> terminaly,
                                               String str) throws
      ZnakyNerozeznanyException, ParamPrazdnyStringException {

    String substr = ""; // pomocny podretezec vstupu
    ArrayList<Znak> retezecZnaku = new ArrayList<Znak> (); // vystup
    int i = 0;

    str = str.trim();
    if (str.equals("")) {
      throw new ParamPrazdnyStringException();
    }
    else {
      if (!str.equals(Znak.jmenoEps)) {
        for (i = 0; i < str.length(); i++) { //pruchod vstupnim Stringem
          substr += str.charAt(i);
          for (Znak neterm : neterminaly) {
            if (neterm.getJmeno().equals(substr)) { //substr je jmeno neterminalu
              retezecZnaku.add(neterm);
              substr = "";
              break;
            }
          }
          if (!substr.equals("")) {
            for (Znak term : terminaly) {
              if (term.getJmeno().equals(substr)) { //substr je jmeno terminalu
                retezecZnaku.add(term);
                substr = "";
                break;
              }
            }
          }
          if (substr.equals(Znak.jmenoEps)) {
            substr = ""; //pohlceni prazdneho slova (epsilonu)
          }
        }
      }
      else {
        try {
          Znak epsilon = new Znak(Znak.jmenoEps, (byte) 0);
          retezecZnaku.add(epsilon);
          str = "";
        }
        catch (ParamPrazdnyStringException ppsex) { //toto nenastane!
          ppsex.printStackTrace();
        }
        catch (PrefixEpsilonuException peex) { //toto nenasatne!
          peex.printStackTrace();
        }
      }

      if (!substr.equals("") && !substr.equals(Znak.jmenoEps)) {
        throw new ZnakyNerozeznanyException();
      }
      else {
        return retezecZnaku;
      }
    }
  }

  /**
   * Metoda first vypot mnoinu FIRST_k zadanho <i>retezce</i>. Vsledn
   * mnoina obsahuje etzce terminl dlky maximln <i>k</i>, na kter se
   * me <i>retezec</i> pepsat za pouit pepisovacch <i>pravidel</i>.
   *
   * @param retezec ArrayList - etzec Znak
   * @param pravidla HashMap - pravidla gramatiky
   * @param k int - parametr <i>k</i>
   * @return HashSet - mnoina etzc Znak (terminl)
   * @throws NekladnyParamException kdy parametr <i>k</i> nen kladn slo
   */
  public static HashSet<ArrayList<Znak>> first(ArrayList<Znak> retezec,
                                               HashMap<Znak,
                                               HashSet<ArrayList<Znak>>>
                                               pravidla,
                                               int k) throws
      NekladnyParamException {

    /* mapy obsahujici dvojice [Neterminal n,first(n)] */
    HashMap<Znak, HashSet<ArrayList<Znak>>> FIPred = //first predchoziho cyklu
        new HashMap<Znak, HashSet<ArrayList<Znak>>> ();
    HashMap<Znak, HashSet<ArrayList<Znak>>> FIAkt = //first aktualniho cyklu
        new HashMap<Znak, HashSet<ArrayList<Znak>>> ();

    Set<Znak> neterminaly = pravidla.keySet(); //mnozina jmen neterminalu

    /* inicializace */
    for (Znak aktNeterm : neterminaly) {
      HashSet<ArrayList<Znak>> mn_prazdna = new HashSet<ArrayList<Znak>> ();
      FIPred.put(aktNeterm, mn_prazdna);
    }
    FIAkt.putAll(FIPred); //na zacatku je v obou mapach to stejne <neterminal,null>

    do {
      FIPred.putAll(FIAkt);

      for (Znak aktNeterm : neterminaly) {
        HashSet<ArrayList<Znak>> aktNetermPravidla = pravidla.get(aktNeterm);

        /* mnozina retezcu, na ktere se muze aktNeterm prepsat */
        HashSet<ArrayList<Znak>> aktNetermFi = new HashSet<ArrayList<Znak>> ();

        for (ArrayList<Znak> aktPravidlo : aktNetermPravidla) {

          /* mnozina retezcu, na ktere se muze aktPravidlo prepsat */
          HashSet<ArrayList<Znak>> aktPravFi = new HashSet<ArrayList<Znak>> ();

          HashSet<ArrayList<Znak>> prvniZnakFi = new HashSet<ArrayList<Znak>> ();
          if (aktPravidlo.get(0).isNeterminal()) {
            prvniZnakFi = FIPred.get(aktPravidlo.get(0));
          }
          else {
            ArrayList<Znak> term_or_eps = new ArrayList<Znak> ();
            term_or_eps.add(aktPravidlo.get(0));
            prvniZnakFi.add(term_or_eps);
          }
          aktPravFi = prvniZnakFi;

          for (int i = 1; i < aktPravidlo.size(); i++) {
            HashSet<ArrayList<Znak>> aktZnakFi = new HashSet<ArrayList<Znak>> ();

            if (aktPravidlo.get(i).isNeterminal()) {
              aktZnakFi = FIPred.get(aktPravidlo.get(i));
            }
            else {
              ArrayList<Znak> term_or_eps = new ArrayList<Znak> ();
              term_or_eps.add(aktPravidlo.get(i));
              aktZnakFi.add(term_or_eps);
            }

            aktPravFi = Retezec.zretez(aktPravFi, aktZnakFi, k);

          } //for

          aktNetermFi.addAll(aktPravFi); //pridam do mnoziny first aktNeterm
        } //for

        FIAkt.put(aktNeterm, aktNetermFi); //vlozim aktNeterm a jeho first mnozinu do FIAkt
      } //for

    }
    while (!FIPred.equals(FIAkt)); //pokracuju, kdyz se FIAkt zmenila

    HashSet<ArrayList<Znak>> vysledek = new HashSet<ArrayList<Znak>> ();

    /* pruchod pres vsechny znaky vstupniho retezce */
    for (Znak aktZnak : retezec) {
      HashSet<ArrayList<Znak>> aktZnakFi = new HashSet<ArrayList<Znak>> ();

      if (aktZnak.isNeterminal()) {
        aktZnakFi = FIPred.get(aktZnak);
      }
      else {
        ArrayList<Znak> term_or_eps = new ArrayList<Znak> ();
        term_or_eps.add(aktZnak);
        aktZnakFi.add(term_or_eps);
      }

      if (!vysledek.isEmpty()) {
        vysledek = Retezec.zretez(vysledek, aktZnakFi, k);
      }
      else {
        vysledek = aktZnakFi;
      }
    }
    return vysledek;
  }

  /**
   * Metoda follow vypot mnoinu FOLLOW_k pro zadan neterminl. Ve vsledn
   * mnoin jsou vechny etzce dlky maximln <i>k</i> takov, kter se
   * mohou po pouit nkterch <i>pravidel</i> gramatiky vyskytovat
   * bezprostedn vpravo od zadanho neterminlu. <br/>
   * Popis algoritmu: <br/>
   * <i>
   *  Je dna gramatika G = (N,E,P,S). <br/>
   *  Postupn potme hodnoty:<br/>
   *    FO_1(A) pro vechny A z N, <br/>
   *    FO_2(A) pro vechny A z N, <br/>
   *    ...                        <br/>
   *    FO_k(A) pro vechny A z N  <br/>
   *<br/>
   *  Pi vpotu FO_i(A) postupujeme nsledovn: <br/>
   *    1. FO_i(S) := {eps} pro poten symbol S, <br/>
   *       FO_i(A) := "przdn mnoina" pro ostatn neterminly <br/>
   *<br/>
   *    2. Pro kad pravidlo tvaru: <br/>
   *          B -> pAq z P, kde p,q jsou etzce a q != eps   <br/>
   *          FO_i(A) = FO_i(A) "sjednoceno s" [(FI_i(q)-{eps}) (+)_i FO_(i-1)(B)]  <br/>
   *<br/>
   *    3. opakuj {    <br/>
   *          pro kad pravidlo tvaru: <br/>
   *             B -> pAq z P, kde p,q jsou etzce a
   *               (q == eps nebo eps je prvkem FI_1(q)) <br/>
   *             FO_i(A) := FO_i(A) "sjednoceno s" FO_i(B) <br/>
   *      }dokud se nedoshne pevnho bodu <br/>
   * </i>
   *
   * @param neterm Znak - neterminl
   * @param pravidla HashMap - mnoina pravidel
   * @param pocatek Znak - poten neterminl gramatiky
   * @param k int - parametr <i>k</i>
   * @return HashSet - mnoina FOLLOW_k pro neterminl <i>neterm</i>
   * @throws NekladnyParamException kdy parametr <i>k</i> je nekladn slo
   */
  public static HashSet<ArrayList<Znak>> follow(Znak neterm, //neterminal
                                                Gramatika gramatika,
                                                int k) throws
      NekladnyParamException {

    if (k >= 1) {

      int i, j, l;
      HashSet<ArrayList<Znak>> vysledek = new HashSet<ArrayList<Znak>> ();

      /*FOLLOW pocitam nad redukovanou gramatikou */
      Gramatika gram_red = gramatika.redukuj();

        /*je-li zredukovana vstupni gramatika neprazdna, pokracuji */
      if (!gram_red.getTerminaly().isEmpty()) {

        HashSet<Znak> neterminaly = gram_red.getNeterminaly(); //mnozina neterminalu
        Znak pocatek = gram_red.getPocatek();                  //pocatecni neterminal
        HashMap<Znak, HashSet<ArrayList<Znak>>> pravidla = gram_red.getPravidla();

        Znak epsilon = null;
        try {
          epsilon = new Znak(Znak.jmenoEps, (byte) 0);
        }
        catch (ParamPrazdnyStringException ex) { //toto nenastane!
          ex.printStackTrace();
        }
        catch (PrefixEpsilonuException peex) { //toto nenastane!
          peex.printStackTrace();
        }

        // retezec obsahujici pouze prazdne epsilon
        ArrayList<Znak> prazdne_slovo = new ArrayList<Znak> ();
        prazdne_slovo.add(epsilon);

        //mnozina obsahujici pouze epsilon
        HashSet<ArrayList<Znak>> mn_epsilon = new HashSet<ArrayList<Znak>> ();
        mn_epsilon.add(prazdne_slovo);

        /* mapy obsahujici dvojice [Neterminal n,follow(n)] */
        HashMap<Znak, HashSet<ArrayList<Znak>>> FOPred =
            new HashMap<Znak, HashSet<ArrayList<Znak>>> ();
        HashMap<Znak, HashSet<ArrayList<Znak>>> FOAkt =
            new HashMap<Znak, HashSet<ArrayList<Znak>>> ();

        for (Znak aktNeterm : neterminaly) {
            HashSet<ArrayList<Znak>> mn_prazdna = new HashSet<ArrayList<Znak>> ();
            FOAkt.put(aktNeterm, mn_prazdna);
        }
        FOAkt.put(pocatek, mn_epsilon); /* inicializace follow(pocatek) */


        /* vypocet follow()_1 az follow()_k */
        for (i = 1; i <= k; i++) {

          //FOPred.putAll(FOAkt);
          FOPred = FOAkt;

          /* 1. FO_i(S) := {eps} pro poten symbol S,
           *    FO_i(A) := "przdn mnoina" pro ostatn neterminly
           */

          for (Znak aktNeterm : neterminaly) {
            HashSet<ArrayList<Znak>> mn_prazdna = new HashSet<ArrayList<Znak>> ();
            FOAkt.put(aktNeterm, mn_prazdna);
          }
          FOAkt.put(pocatek, mn_epsilon); /* inicializace follow(pocatek) */

          /*2. Pro kad pravidlo tvaru:
           *     B -> pAq z P, kde p,q jsou etzce a q != eps
           *     FO_i(A) = FO_i(A) "sjednoceno s" [(FI_i(q)-{eps}) (+)_i FO_(i-1)(B)]
           */

          /* pruchod pres neterminaly */
          for (Znak aktNeterm : neterminaly) {
            HashSet<ArrayList<Znak>>
                aktNetermPravidla = pravidla.get(aktNeterm);
            HashSet<ArrayList<Znak>> aktNetermFo = FOPred.get(aktNeterm);

            for (ArrayList<Znak> aktPravidlo : aktNetermPravidla) {

              /* pruchod znaky aktNetermPravidla az k predposlednimu */
              for (j = 0; j < (aktPravidlo.size() - 1); j++) {
                Znak aktZnak = aktPravidlo.get(j);

                if (aktZnak.isNeterminal()) {
                  ArrayList<Znak> zbytekAktPravidla = new ArrayList<Znak> ();
                  HashSet<ArrayList<Znak>> zbytekAktPravFi;

                  for (l = j + 1; l < aktPravidlo.size(); l++) {
                    zbytekAktPravidla.add(aktPravidlo.get(l));
                  }

                  zbytekAktPravFi = Retezec.first(zbytekAktPravidla,pravidla,i);

                  /* zbytekAktPravFi = (zbytekAktPravFi - {eps}) (+)_k aktNetermFo */
                  zbytekAktPravFi.remove(mn_epsilon);

                  /* pro i=1  se neprovede zretezeni FI_i(q)-{eps} s FO_(i-1)(B)
                     protoze neexistuje FO_i-1(B)
                   */
                  if (i != 1) {
                    zbytekAktPravFi = Retezec.zretez(zbytekAktPravFi,
                        aktNetermFo, i);
                  }
                  HashSet<ArrayList<Znak>> aktZnakFo = new HashSet<ArrayList<Znak>>();
                  aktZnakFo.addAll(FOAkt.get(aktZnak)); //FO(aktZnak)

                  aktZnakFo.addAll(zbytekAktPravFi);
                  FOAkt.put(aktZnak, aktZnakFo);
                } //if

              } //for

            } //for

          } //for

          /*3. opakuj {
           *      pro kad pravidlo tvaru:
           *         B -> pAq z P, kde p,q jsou etzce a
           *              (q == eps nebo eps je prvkem FI_1(q))
           *       FO_i(A) := FO_i(A) "sjednoceno s" FO_i(B)
           *   }dokud se nedoshne pevnho bodu
           */

          do {

            FOPred.putAll(FOAkt);

            for (Znak aktNeterm : neterminaly) {
              HashSet<ArrayList<Znak>>
                  aktNetermPravidla = pravidla.get(aktNeterm);
              HashSet<ArrayList<Znak>> aktNetermFo = FOAkt.get(aktNeterm);

              for (ArrayList<Znak> aktPravidlo : aktNetermPravidla) {

                /* pruchod znaky aktNetermPravidla az k predposlednimu */
                for (j = 0; j < aktPravidlo.size() - 1; j++) {
                  Znak aktZnak = aktPravidlo.get(j);

                  if (aktZnak.isNeterminal()) {
                    ArrayList<Znak> zbytekAktPravidla = new ArrayList<Znak> ();
                    HashSet<ArrayList<Znak>> zbytekAktPravFi;

                    for (l = j + 1; l < aktPravidlo.size(); l++) {
                      zbytekAktPravidla.add(aktPravidlo.get(l)); //zbytekAktPravidla aktNetermPravidla
                    }

                    zbytekAktPravFi = Retezec.first(zbytekAktPravidla,
                        pravidla,
                        1); //first_1(zbytekAktPravidla)

                    if (zbytekAktPravFi.contains(mn_epsilon)) {
                      HashSet<ArrayList<Znak>> aktZnakFo = FOAkt.get(aktZnak); //FO(aktZnak)

                      aktZnakFo.addAll(aktNetermFo);
                      FOAkt.put(aktZnak, aktZnakFo);
                    } //if
                  } //if

                } //for

                Znak aktZnak = aktPravidlo.get(j); //posledni znak v aktPravidle (q == eps)
                if (aktZnak.isNeterminal()) {
                  HashSet<ArrayList<Znak>> aktZnakFo = FOAkt.get(aktZnak); //FO(aktZnak)
                  aktZnakFo.addAll(aktNetermFo);
                  FOAkt.put(aktZnak, aktZnakFo);
                }
              } //for

            } //for

          }
          while (!FOPred.equals(FOAkt)); //opakuj dokud se meni FOAkt

        } //for

        /* je-li neterm dosazitelny, pak je FO_i(netern) je ulozeno v FOAkt,
         * jinak je FO_i(neterm) prazdna mnozina */
        if (neterminaly.contains(neterm)) {
          vysledek.addAll(FOAkt.get(neterm));
        }
      }
      return vysledek;

    }
    else {
      throw new NekladnyParamException();
    }
  }
}
