package webovyformular.dat_typy;

/**
 *
 * <p>Description: </p> Tda Znak slou k reprezentaci terminlnch a
 * neterminlnch symbol gramatiky a przdnho slova - epsilonu. Jsou v n
 * implementovny zkladn metody, pro prci s tmito prvky gramatiky.
 *
 * <p>Copyright: Copyright (c) 2005</p>
 *
 * @author Michal Hanzelka
 * @version 1.0
 */


public class Znak {
  /**
   * Ve statick promnn <i>jmenoEps</i> je uloen etzec, kter se
   * implicitn piad k atributu <i>jmeno</i> pi voln konstruktoru tdy
   * <i>Znak</i> s parametrem typu byte rovnm 0.
   */
  public static String jmenoEps = "eps";

  private String jmeno;
  private byte typ; // 1 = terminal, 2 = neterminal, 0 = epsilon

  /**
   * Konstruktor vytvo nov Znak, jeho atributy budou hodnoty zadan na
   * vstupu. Je-li novou instanc epsilon, jeho jmno bude hodnota veejn
   * statick promnn <i>jmenoEps</i> tdy Znak.
   * @param jmeno String - jmno novho symbolu
   * @param typ byte - typ novho symbolu. Pro terminl <i>typ</i> = 1, pro
   * neterminl <i>typ</i> = 2 a pro epsilon <i>typ</i> = 0.
   * @throws ParamPrazdnyStringException  kdy <i>jmeno</i> je przdn etzec
   * nebo etzec przdnch znak.
   * @throws PrefixEpsilonuException  kdy <i>jmeno</i> je prefixem etzce
   * uloenho v pormnn <i>jmenoEps</i>.
   */
  public Znak(String jmeno, byte typ) throws ParamPrazdnyStringException,
      PrefixEpsilonuException {
    if (!jmeno.trim().equals("")) {
      if (!jmenoEps.startsWith(jmeno) || (typ == 0)) {
        this.jmeno = ( (typ == 0) ? Znak.jmenoEps : jmeno);
        this.typ = typ;
      }
      else {
        System.err.println("Jmno Znaku " + jmeno + " je prefixem " + jmenoEps +
                           ", co je jmno epsilonu.");
        throw new PrefixEpsilonuException();
      }
    }
    else {
      System.err.println("Jmno Znaku neme bt przdn etzec.");
      throw new ParamPrazdnyStringException();
    }
  }

  /**
   * Metoda toString vrac Stringovou reprezentaci danho symbolu, kter odpovd
   * jmnu dan instance.
   * @return String - Stringov reprezentace instance
   */
  public String toString() {
    return jmeno;
  }

  /**
   * Metoda getJmeno vrac jmno instance.
   * @return String - jmno instance
   */
  public String getJmeno() {
    return jmeno;
  }

  /**
   * Metoda getTyp vrac byte hodnotu, kter reprezentuje typ instance. Pro
   * terminl <i>typ</i> = 1, pro neterminl <i>typ</i> = 2 a pro epsilon
   * <i>typ</i> = 0.
   * @return byte - byte hodnota reprezentujc typ instance
   */
  public byte getTyp() {
    return typ;
  }

  /**
   * Metoda hashCode vrac hodnotu hash kodu, kter se vypot jako souet
   * hodnot <i>hashCode() jmena</i> a <i>typu</i>
   * @return int - hodnota hash kodu pro danou instaci
   */
  public int hashCode() {
    return (jmeno.hashCode() + ( (Byte) typ).hashCode());
  }

  /**
   * Metoda equals vrac hodnotu true, pokud je zadan objekt <i>o</i> stejnho
   * jmna a typu jako volajc instance. V opanm ppad metoda vrac false.
   * @param o Object - vstupn objekt
   * @return boolean - true, kdy se objekt a instance shoduj, jinak false
   */
  public boolean equals(Object o) {
    if (o == this) {
      return true;
    }
    if ( (o instanceof Znak) == false) {
      return false;
    }
    Znak zn = (Znak) o;
    return ( (jmeno.equals(zn.getJmeno())) && (typ == zn.getTyp()));
  }

  /**
   * Metoda isTerminal vrac hodnotu true, pokud hodnota atributu <i>typ</i>
   * volajc instance je rovna 1.V opanm ppad vrac false.
   * @return boolean - true, kdy je atribut <i>typ</i> instance roven 1, jinak
   *  false
   */
  public boolean isTerminal() {
    return (1 == getTyp());
  }

  /**
   * Metoda isNeterminal vrac hodnotu true, pokud hodnota atributu <i>typ</i>
   * volajc instance je rovna 2.V opanm ppad vrac false.
   * @return boolean - true, kdy je atribut <i>typ</i> instance roven 2, jinak
   *  false
   */
  public boolean isNeterminal() {
    return (2 == getTyp());
  }

  /**
   * Metoda isEpsilon vrac hodnotu true, pokud hodnota atributu <i>typ</i>
   * volajc instance je rovna 0.V opanm ppad vrac false.
   * @return boolean - true, kdy je atribut <i>typ</i> instance roven 0, jinak
   *  false
   */
  public boolean isEpsilon() {
    return (0 == getTyp());
  }
}
